"use strict";

var _interopRequireDefault = require("@babel/runtime-corejs2/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = exports.QueryBuilder = void 0;

var _parseInt2 = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/parse-int"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime-corejs2/helpers/createClass"));

var _keys = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/object/keys"));

var _isArray = _interopRequireDefault(require("@babel/runtime-corejs2/core-js/array/is-array"));

var _validators = _interopRequireDefault(require("./validators"));

/* eslint no-undefined : 0 */
function buildFilters(queryBuilder, filterSet) {
  var filters = [],
      getMatch = function getMatch(field, value, isPhrase) {
    var filter = {
      match: {}
    };
    isPhrase = isPhrase || false;
    filter.match[field] = {
      query: value
    };

    if (isPhrase) {
      filter.match[field].type = 'phrase';
    } else {
      filter.match[field].operator = 'and';
    }

    return filter;
  },
      getMatches = function getMatches(field, value) {
    if (typeof value === 'string' && /\,/g.test(value)) {
      value = value.split(/\,/g);
    }

    if ((0, _isArray.default)(value)) {
      return value.map(function (item) {
        return getMatch(field, item);
      });
    }

    return [getMatch(field, value)];
  },
      getRange = function getRange(field, value) {
    var filter = {
      range: {}
    };
    filter.range[field] = value;
    return filter;
  },
      getTerm = function getTerm(field, value) {
    var filter = {
      term: {}
    };
    filter.term[field] = value;
    return filter;
  },
      getTerms = function getTerms(field, value) {
    if (typeof value === 'string' && /\,/g.test(value)) {
      value = value.split(/\,/g);
    }

    if ((0, _isArray.default)(value)) {
      return value.map(function (item) {
        return getTerm(field, item);
      });
    }

    return [getTerm(field, value)];
  },
      gt,
      gte,
      lt,
      lte,
      missingFilter;

  if (filterSet.beginsWith) {
    (0, _keys.default)(filterSet.beginsWith).forEach(function (field) {
      if (queryBuilder.matchFields.indexOf(field) >= 0) {
        filters.push(getMatch(field, [filterSet.beginsWith[field]].join('')));
        return;
      }

      filters.push(getTerm(field, [filterSet.beginsWith[field], '*'].join('')));
    });
  }

  if (filterSet.contains) {
    (0, _keys.default)(filterSet.contains).forEach(function (field) {
      if (queryBuilder.matchFields.indexOf(field) >= 0) {
        filters.push(getMatch(field, [filterSet.contains[field]].join('')));
        return;
      }

      filters.push(getTerm(field, ['*', filterSet.contains[field], '*'].join('')));
    });
  }

  if (filterSet.endsWith) {
    (0, _keys.default)(filterSet.endsWith).forEach(function (field) {
      if (queryBuilder.matchFields.indexOf(field) >= 0) {
        filters.push(getMatch(field, [filterSet.endsWith[field]].join('')));
        return;
      }

      filters.push(getTerm(field, ['*', filterSet.endsWith[field]].join('')));
    });
  }

  if (filterSet.exact) {
    (0, _keys.default)(filterSet.exact).forEach(function (field) {
      if (queryBuilder.matchFields.indexOf(field) >= 0) {
        filters = filters.concat(getMatches(field, [filterSet.exact[field]].join(''), true));
        return;
      }

      filters = filters.concat(getTerms(field, filterSet.exact[field]));
    });
  } // handle checks for the existence of a field


  if (filterSet.exists) {
    if (typeof filterSet.exists === 'string' && /\,/g.test(filterSet.exists)) {
      filterSet.exists = filterSet.exists.split(/\,/g);
    }

    if (!(0, _isArray.default)(filterSet.exists)) {
      filterSet.exists = [filterSet.exists];
    }

    filterSet.exists.forEach(function (field) {
      var existsFilter = {
        exists: {
          field: field
        }
      };
      filters.push(existsFilter);
    });
  }

  if (filterSet.greaterThan || filterSet.gt) {
    gt = filterSet.greaterThan || filterSet.gt;
    (0, _keys.default)(gt).forEach(function (field) {
      filters.push(getRange(field, {
        gt: gt[field]
      }));
    });
  }

  if (filterSet.greaterThanEqual || filterSet.gte) {
    gte = filterSet.greaterThanEqual || filterSet.gte;
    (0, _keys.default)(gte).forEach(function (field) {
      filters.push(getRange(field, {
        gte: gte[field]
      }));
    });
  }

  if (filterSet.lessThan || filterSet.lt) {
    lt = filterSet.lessThan || filterSet.lt;
    (0, _keys.default)(lt).forEach(function (field) {
      filters.push(getRange(field, {
        lt: lt[field]
      }));
    });
  }

  if (filterSet.lessThanEqual || filterSet.lte) {
    lte = filterSet.lessThanEqual || filterSet.lte;
    (0, _keys.default)(lte).forEach(function (field) {
      filters.push(getRange(field, {
        lte: lte[field]
      }));
    });
  } // handle checks for the non-existence of a field


  if (filterSet.missing) {
    if (typeof filterSet.missing === 'string' && /\,/g.test(filterSet.missing)) {
      filterSet.missing = filterSet.missing.split(/\,/g);
    }

    if (!(0, _isArray.default)(filterSet.missing)) {
      filterSet.missing = [filterSet.missing];
    }

    filterSet.missing.forEach(function (field) {
      missingFilter = {
        missing: {
          exists: {
            field: field
          }
        }
      };
      filters.push(missingFilter);
    });
  }

  if (filterSet.ne) {
    (0, _keys.default)(filterSet.ne).forEach(function (field) {
      filters.push(getMatch(field, [filterSet.ne[field]].join('')));
    });
  } // return the array of filters to the caller


  return filters;
}

var QueryBuilder =
/*#__PURE__*/
function () {
  function QueryBuilder(matchFields, diagnosticsFilterKey) {
    (0, _classCallCheck2.default)(this, QueryBuilder);
    this.matchFields = matchFields;
    this.diagnosticsFilterKey = diagnosticsFilterKey;
  }

  (0, _createClass2.default)(QueryBuilder, [{
    key: "buildQuery",
    value: function buildQuery(queryOptions) {
      var _this = this;

      var defaultQuery = {
        'match_all': {}
      },
          filter,
          filters = queryOptions.filters,
          hasFilterFilters = false,
          hasQueryFilters = false,
          mustNot,
          query = {
        query: {
          bool: {}
        }
      },
          queryBool,
          sortFields,
          sortOptions = []; // reference bool for simplicity

      queryBool = query.query.bool; // keyword search

      if (filters && filters.keyword) {
        filter = {
          match: filters.keyword
        };
        queryBool.must = (queryBool.must || []).concat(filter);
      } // add analytics filters to the query


      if (filters && filters.diagnostics && this.diagnosticsFilterKey) {
        if (!(0, _isArray.default)(filters.diagnostics)) {
          if (/\,/g.test(filters.diagnostics)) {
            filters.diagnostics = filters.diagnostics.split(/\,/);
          } else {
            filters.diagnostics = [filters.diagnostics];
          }
        }

        queryBool.filter = queryBool.filter || {};
        filters.diagnostics.forEach(function (stat) {
          if (typeof _this.diagnosticsFilterKey[stat] !== 'undefined') {
            queryBool.filter.terms = (queryBool.filter.terms || []).concat(_this.diagnosticsFilterKey[stat]());
          }
        });
      } // add mandatory filters to the query


      if (filters && filters.mandatory) {
        // notEqual queries need to be processed differently than normal
        // match queries
        if (filters.mandatory.ne || filters.mandatory.notEqual || filters.mandatory.notEqualTo) {
          mustNot = {
            'ne': filters.mandatory.ne || filters.mandatory.notEqual || filters.mandatory.notEqualTo
          }; // remove notEqual parameters so they aren't processed again

          filters.mandatory.ne = undefined;
          filters.mandatory.notEqual = undefined;
          filters.mandatory.notEqualTo = undefined;
          buildFilters(this, mustNot).forEach(function (filter) {
            // right now, only exact match is support
            if (filter.match) {
              queryBool['must_not'] = (queryBool['must_not'] || []).concat(filter);
              return;
            }
          });
        }

        buildFilters(this, filters.mandatory).forEach(function (filter) {
          if (filter.match) {
            queryBool.must = (queryBool.must || []).concat(filter);
            return;
          }

          if (filter.missing) {
            queryBool['must_not'] = (queryBool['must_not'] || []).concat(filter.missing);
            return;
          }

          queryBool.must = (queryBool.must || []).concat(filter);
        });
      } // add optional filters to the query


      if (filters && filters.optional) {
        buildFilters(this, filters.optional).forEach(function (filter) {
          if (filter.match) {
            queryBool.should = ((0, _isArray.default)(queryBool.should) ? queryBool.should : []).concat(filter);
            return;
          }

          queryBool.should = ((0, _isArray.default)(queryBool.should) ? queryBool.should : []).concat(filter);
        });
      } // add projection


      if (filters && filters.field) {
        // ensure array
        if (!(0, _isArray.default)(filters.field)) {
          if (/\,/g.test(filters.field)) {
            filters.field = filters.field.split(/\,/);
          } else {
            filters.field = [filters.field];
          }
        }

        query._source = (query._source || []).concat(filters.field);
      }

      if (queryOptions.sort) {
        if (typeof queryOptions.sort === 'string') {
          sortOptions = queryOptions.sort.split(/\,/g).map(function (field) {
            return field.trim();
          });
        }

        if ((0, _isArray.default)(queryOptions.sort) && queryOptions.sort.length) {
          sortOptions = queryOptions.sort;
        }

        if (_validators.default.isPOJO(queryOptions.sort)) {
          (0, _keys.default)(queryOptions.sort).forEach(function (property) {
            // support for more traditional way of supplying asc params
            if (property === 'asc') {
              sortFields = (0, _isArray.default)(queryOptions.sort.asc) ? queryOptions.sort.asc : [queryOptions.sort.asc];
              sortOptions = sortOptions.concat(sortFields);
              return;
            } // support for more traditional way of supplying desc params


            if (property === 'desc') {
              sortFields = (0, _isArray.default)(queryOptions.sort.desc) ? queryOptions.sort.desc : [queryOptions.sort.desc];
              sortFields = sortFields.map(function (field) {
                return '-' + field;
              });
              sortOptions = sortOptions.concat(sortFields);
              return;
            }

            if (queryOptions.sort[property] === 'desc' || (0, _parseInt2.default)(queryOptions.sort[property], 10) < 0) {
              sortOptions.push('-' + property);
              return;
            }

            sortOptions.push(property);
          });
        }

        query.sort = query.sort || [];
        sortOptions.forEach(function (field) {
          var sortValue = {};

          if (field.startsWith('-')) {
            sortValue[field.substring(1)] = 'desc';
          } else {
            sortValue[field] = 'asc';
          }

          query.sort.push(sortValue);
        });
      } // add pagination details


      if (typeof queryOptions.start !== 'undefined' && !isNaN(queryOptions.start)) {
        query.from = (0, _parseInt2.default)(queryOptions.start, 10);
      }

      if (typeof queryOptions.count !== 'undefined' && !isNaN(queryOptions.count)) {
        query.size = (0, _parseInt2.default)(queryOptions.count, 10);
      } // if no query booleans were applied, remove query


      hasQueryFilters = queryBool.should && queryBool.should.length > 0 || queryBool.must && queryBool.must.length > 0 || queryBool.must_not && queryBool.must_not.length > 0;

      if (!hasQueryFilters) {
        query.query.bool.must = undefined;
      } // if no filter booleans were applied, remove filter


      hasFilterFilters = queryBool.filter && queryBool.filter.length > 0;

      if (!hasFilterFilters) {
        query.query.bool.filter = undefined;
      }

      if (!hasFilterFilters && !hasQueryFilters) {
        query.query = defaultQuery;
      } // return to caller


      return query;
    }
  }]);
  return QueryBuilder;
}();

exports.QueryBuilder = QueryBuilder;
var _default = QueryBuilder;
exports.default = _default;
//# sourceMappingURL=builder.js.map
